<?php
/**
 * Plugin Name: Advanced URL Coupons
 * Description: Generate URL coupons that automatically apply discounts along with specific products to the cart when a customer visits your store using the coupon-link.
 * Plugin URI:https://woocommerce.com/products/addify-advanced-url-coupons/
 * Description: Addify Url Coupon
 * Version: 1.1.1
 * Author: Addify
 * Developed By: Addify
 * Author URI:https://woocommerce.com/vendor/addify/
 * Support:https://woocommerce.com/vendor/addify/
 * License:           GNU General Public License v3.0
 * License URI:       http://www.gnu.org/licenses/gpl-3.0.html
 * Domain Path: /languages
 * TextDomain : addify-advanced-url-coupons
 * WC requires at least: 4.0
	* WC tested up to: 9.*.*
	* Requires at least: 6.5
	* Tested up to: 6.6.*
	* Requires PHP: 7.4
 * /**
 * Define class.
 */


if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class AF_WUC_Main {

	public function __construct() {
		$this->af_wuc_global_constant_variables();

		add_action( 'plugins_loaded', array( $this, 'af_wuc_check_woocommerce' ) );
		add_action( 'init', array( $this, 'af_wuc_hooks' ) );
		add_action( 'woocommerce_blocks_loaded', array( $this, 'ka_up_block_init' ) );
	}//end __construct()

	public function af_wuc_global_constant_variables() {
		if ( ! defined( 'AF_WUC_URL' ) ) {
			define( 'AF_WUC_URL', plugin_dir_url( __FILE__ ) );
		}
		if ( ! defined( 'AF_WUC_BASENAME' ) ) {
			define( 'AF_WUC_BASENAME', plugin_basename( __FILE__ ) );
		}
		if ( ! defined( 'AF_WUC_PLUGIN_DIR' ) ) {
			define( 'AF_WUC_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
		}
	}//end af_wuc_global_constant_variables()

	public function af_wuc_check_woocommerce() {
		// Check the installation of WooCommerce module if it is not a multi site.
		if ( ! is_multisite() && ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ), true ) ) ) {
			add_action( 'admin_notices', array( $this, 'addify_wuc_admin_notice' ) );
		}
	}
	public function addify_wuc_admin_notice() {
		deactivate_plugins( __FILE__ );
		?>
			<div id="message" class="error">
				<p>
					<strong>
					<?php esc_html_e( 'Advanced URL Coupons plugin is inactive. WooCommerce plugin must be active in order to activate it.', 'addify-advanced-url-coupons' ); ?>
					</strong>
				</p>
			</div>
			<?php
	}

	public function af_wuc_hooks() {
		if ( defined( 'WC_PLUGIN_FILE' ) ) {
			add_action( 'after_setup_theme', array( $this, 'af_wuc_reg_init' ) );
			add_action( 'before_woocommerce_init', array( $this, 'af_wuc_hops_compatibility' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'af_wuc_enqueue_select2_jquery' ) );
			add_action( 'wp_loaded', array( $this, 'af_wuc_custom_post_type' ) );

			add_filter( 'plugin_action_links_' . AF_WUC_BASENAME, array( $this, 'af_wuc_plugin_action_links' ) );
				include_once AF_WUC_PLUGIN_DIR . '/include/af-wuc-general-functions.php';

			if ( is_admin() ) {
				include_once AF_WUC_PLUGIN_DIR . '/include/admin/class-af-wuc-admin.php';
			} else {
				include_once AF_WUC_PLUGIN_DIR . '/include/front/class-af-wuc-front.php';
			}
		}
	}


	/**
	 * Add custom action links on plugin screen.
	 *
	 * @param mixed $actions Plugin Actions Links.
	 * @return array
	 */
	public function af_wuc_plugin_action_links( $actions ) {
		$custom_actions = array(
			'settings' => sprintf( '<a href="%s">%s</a>', admin_url( 'admin.php?page=wc-settings&tab=woocommerce_url_coupon_tab' ), __( 'Settings', 'addify-advanced-url-coupons' ) ),
		);
		return array_merge( $custom_actions, $actions );
	}

	public function af_wuc_custom_post_type() {

		$labels   = array(
			'name'               => esc_html__( 'URL Coupon', 'addify-advanced-url-coupons' ),
			'singular_name'      => esc_html__( 'URL Coupon', 'addify-advanced-url-coupons' ),
			'all_items'          => esc_html__( 'URL Coupons', 'addify-advanced-url-coupons' ),
			'add_new'            => esc_html__( 'Add New ', 'addify-advanced-url-coupons' ),
			'add_new_item'       => esc_html__( 'Add New URL Coupon', 'addify-advanced-url-coupons' ),
			'edit_item'          => esc_html__( 'Edit URL Coupon', 'addify-advanced-url-coupons' ),
			'new_item'           => esc_html__( 'URl Coupon', 'addify-advanced-url-coupons' ),
			// 'view_item'          => esc_html__( 'View Url Coupon', 'addify-advanced-url-coupons' ),
				'search_items'   => esc_html__( 'Search Url Coupon', 'addify-advanced-url-coupons' ),
			'not_found'          => esc_html__( 'No Url Coupon', 'addify-advanced-url-coupons' ),
			'not_found_in_trash' => esc_html__( 'No Url Coupon found in trash', 'addify-advanced-url-coupons' ),
			'parent_item_colon'  => '',
			'menu_name'          => esc_html__( 'URL Coupons', 'addify-advanced-url-coupons' ),
			'item_published'     => esc_html__( 'URL Coupon published', 'addify-advanced-url-coupons' ),
			'item_updated'       => esc_html__( 'URL Coupon updated', 'addify-advanced-url-coupons' ),
			'attributes'         => esc_html__( 'Rule Priority', 'addify-advanced-url-coupons' ),
		);
		$supports = array( 'title' );
		$options  = array(
			'labels'            => $labels,
			'public'            => true,
			'publicly_querable' => true,
			'query_var'         => true,
			'can_export'        => true,
			'show_ui'           => true,
			'show_in_menu'      => 'woocommerce-marketing',
			'show_in_admin_bar' => false,
			'show_in_nav_menus' => false,
			'has_archive'       => true,
			'rewrite'           => array(
				'slug',
				'addify-url-coupon',
			),
			'show_in_rest'      => true,
			'supports'          => $supports, // Empty array disables all input fields
			'map_meta_cap'      => true,

		);
		register_post_type( 'af_wuc', $options );
	
		$labels   = array(
			'name'               => esc_html__( 'URL Coupon Order', 'addify-advanced-url-coupons' ),
			'singular_name'      => esc_html__( 'URL Coupon Order', 'addify-advanced-url-coupons' ),
			'all_items'          => esc_html__( 'URL Coupon Orders', 'addify-advanced-url-coupons' ),
			'add_new_item'       => esc_html__( 'Add new URL Coupon Order', 'addify-advanced-url-coupons' ),
			'edit_item'          => esc_html__( 'Edit URl Coupon  Order', 'addify-advanced-url-coupons' ),
			'new_item'           => esc_html__( 'Url Coupon Order', 'addify-advanced-url-coupons' ),
			'view_item'          => esc_html__( 'View Url Coupon Order', 'addify-advanced-url-coupons' ),
			'search_items'       => esc_html__( 'Search Url Coupon Order', 'addify-advanced-url-coupons' ),
			'not_found'          => esc_html__( 'No Url Coupon Order', 'addify-advanced-url-coupons' ),
			'not_found_in_trash' => esc_html__( 'No Url Coupon Order found in trash', 'addify-advanced-url-coupons' ),
			'parent_item_colon'  => '',
			'menu_name'          => esc_html__( 'Url Coupon Orders', 'addify-advanced-url-coupons' ),
			'item_published'     => esc_html__( 'Url Coupon Order published', 'addify-advanced-url-coupons' ),
			'item_updated'       => esc_html__( 'Url Coupon Order updated', 'addify-advanced-url-coupons' ),
			'attributes'         => esc_html__( 'Rule Priority', 'addify-advanced-url-coupons' ),
		);
		$supports = array( 'title' );
		$options  = array(
			'labels'            => $labels,
			'public'            => true,
			'publicly_querable' => true,
			'query_var'         => true,
			'can_export'        => true,
			'show_ui'           => true,
			'show_in_menu'      => false,
			'show_in_admin_bar' => false,
			'show_in_nav_menus' => false,
			'has_archive'       => true,
			'rewrite'           => array(
				'slug',
				'addify-url-coupon-orders',
			),
			'show_in_rest'      => true,
			'supports'          => $supports,
			'map_meta_cap'      => true,

		);
		register_post_type( 'af_wuc_orders', $options );
	}

	public function af_wuc_enqueue_select2_jquery() {
		wp_enqueue_style( 'select2-css', plugins_url( 'assets/css/select2.css', WC_PLUGIN_FILE ), array(), '5.7.2' );
		wp_enqueue_script( 'select2-js', plugins_url( 'assets/js/select2/select2.min.js', WC_PLUGIN_FILE ), array( 'jquery' ), '4.0.3', true );
	}

	public function af_wuc_hops_compatibility() {

		if ( class_exists( Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
			Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
		}
	}

	public function af_wuc_reg_init() {

		if ( function_exists( 'load_plugin_textdomain' ) ) {
			load_plugin_textdomain( 'addify-advanced-url-coupons', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}
	}
	public function ka_up_block_init() {

		// if ( ( $this->ka_gw_woocommerce_blocks_active() )&&( $this->ka_gw_woocommerce_blocks_version_supported() ) ) {

		require_once AF_WUC_PLUGIN_DIR . '/include/af-wuc-functions.php';

		if ( class_exists( 'AF_WUC_Block_Class' ) ) {
			add_action(
			'woocommerce_blocks_cart_block_registration',
				function () {
				new AF_WUC_Block_Class();
				}
			);
		}
	}
}//end class


if (class_exists('AF_WUC_Main')) {
	new AF_WUC_Main();
}

