<?php

if (!defined('ABSPATH')) {
	exit;
}




class AF_WUC_Admin {



	public function __construct() {
		add_action('admin_enqueue_scripts', array( $this, 'af_wuc_meta_box_fields_style' ));

		add_filter('woocommerce_settings_tabs_array', array( $this, 'af_wuc_add_settings_tab' ), 50);
		add_action('woocommerce_sections_woocommerce_url_coupon_tab', array( $this, 'af_wuc_get_sections' ));
		add_action('woocommerce_settings_woocommerce_url_coupon_tab', array( $this, 'af_wuc_settings_tab' ));
		add_action('woocommerce_settings_save_woocommerce_url_coupon_tab', array( $this, 'af_wuc_update_settings' ));

		add_action('add_meta_boxes', array( $this, 'af_wuc_custom_metabox' ));
		add_action('save_post_af_wuc', array( $this, 'save_post_af_wuc_function' ));

		add_action('wp_ajax_af_wuc_product_search', array( $this, 'af_wuc_live_product_search' ));
		add_action('wp_ajax_af_wuc_get_product_attributes', array( $this, 'af_wuc_get_product_attributes' ));
		add_action('wp_ajax_af_wuc_get_stock_quantity', array( $this, 'af_wuc_get_stock_quantity' ));

		add_filter('manage_af_wuc_posts_columns', array( $this, 'af_wuc_add_custom_columns_post_type' ));
		add_action('manage_af_wuc_posts_custom_column', array( $this, 'af_wuc_populate_data_custom_columns' ), 10, 2);
		add_action('pre_get_posts', array( $this, 'af_wuc_custom_column_sorting' ));
		add_filter('manage_edit-af_wuc_sortable_columns', array( $this, 'af_wuc_custom_columns_sortable' ));

		add_filter('post_updated_messages', array( $this, 'af_wuc_custom_post_updated_messages' ));

		add_filter('post_row_actions', array( $this, 'af_wuc_row_actions' ), 10, 2);
		add_filter('page_row_actions', array( $this, 'af_wuc_row_actions' ), 10, 2);

		add_action('admin_head', array( $this, 'af_wuc_custom_admin_post_css' ));

		add_action('woocommerce_coupon_options', array( $this, 'add_custom_coupon_field' ));

		add_action('woocommerce_process_shop_coupon_meta', array( $this, 'save_custom_coupon_field' ));

		add_filter('manage_edit-shop_coupon_columns', array( $this, 'add_custom_coupon_column' ));
		add_action('manage_shop_coupon_posts_custom_column', array( $this, 'populate_custom_coupon_column' ), 10, 2);
		add_action('admin_init', array( $this, 'remove_post_title_field' ));
	}//end __construct()

	public function remove_post_title_field() {

	$today = time();

		$af_wuc_coupon_posts = get_posts(
			array(
				'orderby'     => 'name',
				'order'       => 'asc',
				'post_type'   => 'shop_coupon',
				'post_status' => 'publish',
				'numberposts' => -1,
			)
		);

		$non_expired_coupons = array();

		foreach ($af_wuc_coupon_posts as $coupon_post) {
			$expiration_date = get_post_meta($coupon_post->ID, 'date_expires', true);
			if (empty($expiration_date) || $expiration_date >= $today) {
				$non_expired_coupons[] = $coupon_post;
			}
		}

		if (count($non_expired_coupons) <1 ) {
			remove_post_type_support('af_wuc', 'title');
			
		}
	}
	/**
	 *  Custom_admin_post_css function start.
	 */
	public function af_wuc_custom_admin_post_css() {

		global $post_type;

		if ('af_wuc' === $post_type) {
			echo '<style>#edit-slug-box {display:none;} #preview-action {display:none;} #visibility {display:none;}</style>';
		}
	}
	public function add_custom_coupon_column( $columns ) {
		$new_columns = array();

		foreach ($columns as $key => $value) {
			if ('expiry_date' === $key) {
				$new_columns['custom_column'] = __('URL Coupons', 'addify-advanced-url-coupons');
			}
			$new_columns[ $key ] = $value;
		}


		return $new_columns;
	}

	public function add_custom_coupon_field( $post_id ) {
		$general_prefix = !empty(get_option('wc_url_general_prefix')) ? get_option('wc_url_general_prefix') : 'af_wuc';

		$af_wuc_prefix = !empty(get_post_meta($post_id, 'af_wuc_prefix', true)) ? get_post_meta($post_id, 'af_wuc_prefix', true) : $general_prefix;

		// Ensure $af_wuc_prefix is a string and not an array
		$af_wuc_prefix = is_array($af_wuc_prefix) ? '' : $af_wuc_prefix;

		if ('checkout_page' == get_option('wc_url_coupon_tab_redirect_to_page')) {
			$home_url = wc_get_page_permalink('checkout');
		} else {
			$home_url = wc_get_page_permalink('cart');

		}

		// Build the URL using both prefixes
		$full_url  = !str_contains($home_url, '?') ? "$home_url?$af_wuc_prefix" : "$home_url&$af_wuc_prefix";
		$full_url .= '-' . $post_id; // Append post ID to make the URL unique

		// Add nonce field for verification
		wp_nonce_field('af_wuc_nonce_action', 'af_wuc_nonce');

		// Display the input field for existing coupons
		woocommerce_wp_text_input(array(
			'id'          => 'af_wuc_prefix',
			'label'       => __('Coupon URL', 'addify-advanced-url-coupons'),
			'placeholder' => esc_attr($general_prefix), // Corrected placeholder value
			'description' => '',
			'desc_tip'    => false, // Enable tooltip for description
			'value'       => is_string($af_wuc_prefix) ? esc_attr($af_wuc_prefix) : '', // Ensure $af_wuc_prefix is a string
		));

		if (!empty($full_url)) {
			echo '<div style="text-align: center; width: 74%; margin-bottom: 20px">';
			echo '<a href="' . esc_url($full_url) . '" target="_blank">' . esc_html($full_url) . '</a>';
			echo '</div>';
		}

		// Save the full URL in post meta to be accessed later if needed
		update_post_meta($post_id, 'af_wuc_coupon_url', esc_url($full_url));
	}


	public function save_custom_coupon_field( $post_id ) {
		// Check if nonce is set and valid
		if ( ! isset( $_POST['af_wuc_nonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['af_wuc_nonce'] ) ), 'af_wuc_nonce_action' ) ) {
			return;
		}





		// Ensure user has the proper permissions to edit the coupon
		if (!current_user_can('edit_post', $post_id)) {
			return;
		}

		// Check if the af_wuc_prefix is set in the request
		if (isset($_POST['af_wuc_prefix'])) {
			$af_wuc_prefix = sanitize_text_field(wp_unslash($_POST['af_wuc_prefix']));
			update_post_meta($post_id, 'af_wuc_prefix', $af_wuc_prefix); // Save or update the meta field

			// Rebuild and save the full coupon URL as well
			$home_url  = home_url();
			$full_url  = !str_contains($home_url, '?') ? "$home_url?$af_wuc_prefix" : "$home_url&$af_wuc_prefix";
			$full_url .= '-' . $post_id; // Append post ID to make the URL unique

			// Save the full URL in post meta
			update_post_meta($post_id, 'af_wuc_coupon_url', esc_url($full_url));
		}
	}


	public function populate_custom_coupon_column( $column, $post_id ) {
		// Check if we are populating the correct column
		if ('custom_column' !== $column) {
			return;
		}
		$af_wuc_post_status = get_post_status($post_id);

		if ('publish' === $af_wuc_post_status) {
			$af_wuc_post_id = get_the_ID();
			$general_prefix = !empty(get_option('wc_url_general_prefix')) ? get_option('wc_url_general_prefix') : 'af_wuc';

			$af_wuc_prefix = !empty(get_post_meta($af_wuc_post_id, 'af_wuc_prefix', true)) ? get_post_meta($af_wuc_post_id, 'af_wuc_prefix', true) : $general_prefix;

			// Ensure $af_wuc_prefix is a string and not an array
			$af_wuc_prefix = is_array($af_wuc_prefix) ? '' : $af_wuc_prefix;

			if ('checkout_page' == get_option('wc_url_coupon_tab_redirect_to_page')) {
				$home_url = wc_get_page_permalink('checkout');
			} else {
				$home_url = wc_get_page_permalink('cart');

			}

			// Build the URL using both prefixes
			$full_url  = !str_contains($home_url, '?') ? "$home_url?$af_wuc_prefix" : "$home_url&$af_wuc_prefix";
			$full_url .= '-' . $post_id; // Append post ID to make the URL unique

			?>

			<a href="<?php echo esc_url($full_url); ?>" target="_blank" id="<?php echo esc_attr($general_prefix); ?>_url">
				<?php echo esc_url($full_url); ?>
			</a>
			<?php

		}
	}



	public function af_wuc_row_actions( $actions, $post ) {
		global $post_type;



		if ('af_wuc' === $post_type) {

			unset($actions['inline hide-if-no-js'], $actions['view']);
		}
		return $actions;
	}

	public function af_wuc_custom_post_updated_messages( $messages ) {
		global $post;
		if ('af_wuc' === $post->post_type) {
			$messages['post'][1]  = esc_html__('Post Updated', 'addify-advanced-url-coupons');
			$messages['post'][6]  = esc_html__('Post Updated', 'addify-advanced-url-coupons');
			$messages['post'][8]  = esc_html__('Post Updated', 'addify-advanced-url-coupons');
			$messages['post'][10] = esc_html__('Post Updated', 'addify-advanced-url-coupons');
		}
		return $messages;
	}



	public function af_wuc_custom_columns_sortable( $sortable_columns ) {
		$sortable_columns['coupon_name'] = 'coupon_name';
		$sortable_columns['orders']      = 'orders';
		$sortable_columns['af_wuc_url']  = 'af_wuc_url';
		return $sortable_columns;
	}

	public function af_wuc_custom_column_sorting( $query ) {
		$orderby = $query->get('orderby');
		if ('coupon_name' === $orderby) {
			$query->set('meta_key', 'af_wuc_coupon');
			$query->set('orderby', 'meta_value');
		} elseif ('af_wuc_url' === $orderby) {
			$query->set('meta_key', 'af_wuc_url');
			$query->set('orderby', 'meta_value');
		} elseif ('orders' === $orderby) {
			$query->set('meta_key', 'orders');
			$query->set('orderby', 'meta_value');
		}
	}


	public function af_wuc_populate_data_custom_columns( $column_name, $post_id ) {
		$coupon_code = get_post_meta($post_id, 'af_wuc_coupon', true);

		if ('coupon_name' === $column_name) {
			$coupon_details = new WC_Coupon($coupon_code);
			echo esc_attr($coupon_details->get_code());

		} elseif ('orders' === $column_name) {
			$args = array(
				'post_parent' => $post_id,
				'post_type'   => 'af_wuc_orders',
			);

			$af_wuc_orders_posts_array = get_children($args);
			$order_count               = 0;
			$order_tr                  = '';

			foreach ($af_wuc_orders_posts_array as $af_wuc_order_post_key => $af_wuc_order_post) {
				$af_wuc_order_id = get_post_meta($af_wuc_order_post_key, 'af_wuc_order_id', true);
				$order           = wc_get_order($af_wuc_order_id);
				++$order_count;

				$order_status       = ucfirst($order->get_status());
				$order_created_date = gmdate('F j, Y g:i a', strtotime($order->get_date_created()));
				$time_diff          = human_time_diff($order->get_date_created()->getTimestamp()) . ' ago';

				$order_tr .= '<tr class="">
			        <td>' . esc_attr($order->get_id()) . '</td>
			        <td><span class="woocommerce-Price-amount amount">
			            <span class="woocommerce-Price-currencySymbol">' . esc_attr(get_woocommerce_currency_symbol()) . '</span>' . esc_attr($order->get_total()) . '
			        </span></td>
			        <td class="order_status column-order_status" data-colname="Status">
			            <span>' . esc_html($order_status) . '</span>
			        </td>
			        <td>
			            <span title="' . esc_attr($order_created_date) . '">' . esc_html($time_diff) . '</span>
			        </td>
			    </tr>';
			}

			echo esc_html( $order_count );
			if ($order_count) {
				?>
				<i class="fa fa-list af_wuc_check_orders"></i>
				<section tabindex="0" class="af_wuc_order_table af_wuc_hide">
					<div class="af_wuc_order_table_modal_header">
						<h3><?php echo esc_html__('Orders', 'addify-advanced-url-coupons'); ?></h3>
						<button class="af_wuc_order_table_btn_close" type="button"><i class="fa fa-close"></i></button>
					</div>
					<div class="af_wuc_order_table_modal_body">
						<table class="wp-list-table widefat fixed table-view-list ">
							<thead>
								<th><b><?php echo esc_html__('Order Id', 'addify-advanced-url-coupons'); ?></b></th>
								<th><b><?php echo esc_html__('Total Bill', 'addify-advanced-url-coupons'); ?></b></th>
								<th><b><?php echo esc_html__('Order Status', 'addify-advanced-url-coupons'); ?></b></th>
								<th><b><?php echo esc_html__('Created Date', 'addify-advanced-url-coupons'); ?></b></th>
							</thead>
							<tbody id="the-list">
								<?php echo wp_kses_post($order_tr); ?>
							</tbody>
						</table>
					</div>
				</section>
				<div class="af_wuc_order_table_overlay  af_wuc_hide"></div>
				<?php
			}

		} elseif ('af_wuc_url' === $column_name) {
			$af_wuc_post_status = get_post_status($post_id);

			if ('publish' === $af_wuc_post_status) {
				$af_wuc_post_id = get_the_ID();
				$general_prefix = !empty(get_option('wc_url_general_prefix')) ? get_option('wc_url_general_prefix') : 'af_wuc';

				$af_wuc_prefix = !empty(get_post_meta($af_wuc_post_id, 'af_wuc_prefix', true)) ? get_post_meta($af_wuc_post_id, 'af_wuc_prefix', true) : $general_prefix;

				// Ensure $af_wuc_prefix is a string and not an array
				$af_wuc_prefix = is_array($af_wuc_prefix) ? '' : $af_wuc_prefix;

				if ('checkout_page' == get_option('wc_url_coupon_tab_redirect_to_page')) {
					$home_url = wc_get_page_permalink('checkout');
				} else {
					$home_url = wc_get_page_permalink('cart');

				}
				$prefix = !empty($af_wuc_prefix) ? $af_wuc_prefix . '-' : $general_prefix;

				$full_url  = !str_contains($home_url, '?') ? "$home_url?$prefix" : "$home_url&$prefix";
				$full_url .= ( !empty($af_wuc_prefix) ? '' : '-' ) . $af_wuc_post_id;
				?>

				<a href="<?php echo esc_url($full_url); ?>" target="_blank" id="<?php echo esc_attr($general_prefix); ?>_url">
					<?php echo esc_url($full_url); ?>
				</a>

				<?php
			} else {
	// Translators: %s is the post status of the URL.
echo esc_html( sprintf( __( 'URL is not available as the post status is %s', 'addify-advanced-url-coupons' ), $af_wuc_post_status ) );
			}


		}
	}

	public function af_wuc_add_custom_columns_post_type( $columns ) {
		$date_column = $columns['date'];
		unset($columns['date']);
		$columns['coupon_name'] = esc_html__('Coupon', 'addify-advanced-url-coupons');
		$columns['orders']      = esc_html__('Orders', 'addify-advanced-url-coupons');
		$columns['af_wuc_url']  = esc_html__('Url', 'addify-advanced-url-coupons');
		$columns['date']        = $date_column;

		return $columns;
	}


	public function af_wuc_custom_metabox() {

		$today = time();

		$af_wuc_coupon_posts = get_posts(
			array(
				'orderby'     => 'name',
				'order'       => 'asc',
				'post_type'   => 'shop_coupon',
				'post_status' => 'publish',
				'numberposts' => -1,
			)
		);

		$non_expired_coupons = array();

		foreach ($af_wuc_coupon_posts as $coupon_post) {
			$expiration_date = get_post_meta($coupon_post->ID, 'date_expires', true);
			if (empty($expiration_date) || $expiration_date >= $today) {
				$non_expired_coupons[] = $coupon_post;
			}
		}

		if (count($non_expired_coupons)) {
			add_meta_box('af_wuc_data', esc_html__('URl Coupon Settings', 'addify-advanced-url-coupons'), array( $this, 'af_wuc_metabox_field' ), 'af_wuc');
			add_meta_box('af_wuc_data_message', esc_html__('Cart Message', 'addify-advanced-url-coupons'), array( $this, 'af_wuc_metabox_message' ), 'af_wuc'); 
		} else {
			add_meta_box('af_wuc_coupon', esc_html__('Coupons', 'addify-advanced-url-coupons'), array( $this, 'af_wuc_metabox_coupon_error' ), 'af_wuc');
		}
	}//end af_wuc_custom_metabox()

	public function save_post_af_wuc_function( $post_id ) {
		if (
			( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) ||
			!current_user_can('edit_posts') ||
			in_array(get_post_status($post_id), array( 'auto-draft', 'trash' ), true) ||
			is_ajax() ||
			'untrash' === ( isset($_GET['action']) ? sanitize_text_field(wp_unslash($_GET['action'])) : '' )
		) {
			return;
		}

		// Verify nonce
		$nonce = isset($_POST['af_wuc_nonce']) ? sanitize_text_field(wp_unslash($_POST['af_wuc_nonce'])) : '';
		if (!wp_verify_nonce($nonce, 'af_wuc_nonce_action')) {
			wp_die('Security Violation');
		}

		// Save products quantity
		$af_wuc_products_quantity = isset($_POST['af_wuc_products_quantity']) ? sanitize_meta('', wp_unslash($_POST['af_wuc_products_quantity']), '') : array();
		update_post_meta($post_id, 'af_wuc_products_quantity', $af_wuc_products_quantity);
		$af_wuc_prefix = isset($_POST['af_wuc_prefix']) ? sanitize_meta('', wp_unslash($_POST['af_wuc_prefix']), '') : array();
		update_post_meta($post_id, 'af_wuc_prefix', $af_wuc_prefix);

		// Save product attributes
		$af_wuc_product_attributes = isset($_POST['af_wuc_product_attribute']) ? sanitize_meta('', wp_unslash($_POST['af_wuc_product_attribute']), '') : array();
		update_post_meta($post_id, 'af_wuc_product_attribute', $af_wuc_product_attributes);

		$af_wuc_variation_attributes = array();
		if (count($af_wuc_product_attributes)) {
			foreach ($af_wuc_product_attributes as $product_id => $af_wuc_single_product_attribute) {
				foreach ($af_wuc_single_product_attribute as $attribute_key => $attribute_value) {
					$af_wuc_variation_attributes[ $product_id ]['attributes'][ 'attribute_' . strtolower($attribute_key) ] = $attribute_value;
				}
				$af_wuc_variation_attributes[ $product_id ]['variation_id'] = $this->af_wuc_get_product_variation_id($product_id, $af_wuc_variation_attributes[ $product_id ]['attributes']);
			}
		}

		if (count($af_wuc_variation_attributes)) {
			update_post_meta($post_id, 'af_wuc_product_variation', $af_wuc_variation_attributes);
		}


			// Check if af_wuc_coupon_url_message is set in the POST data
			$af_wuc_coupon_url_message = isset($_POST['af_wuc_coupon_url_message']) 
				? wp_kses_post(wp_unslash($_POST['af_wuc_coupon_url_message'])) 
				: '';

			// Update post meta with sanitized content
			update_post_meta($post_id, 'af_wuc_coupon_url_message', $af_wuc_coupon_url_message);


		$af_wuc_coupon = isset($_POST['af_wuc_coupon']) ? sanitize_text_field(wp_unslash($_POST['af_wuc_coupon'])) : '';
		update_post_meta($post_id, 'af_wuc_coupon', $af_wuc_coupon);

		// Set default coupon URL
		$get_af_wuc_url = get_post_meta($post_id, 'af_wuc_url', true);
		if (empty($get_af_wuc_url)) {
			$af_wuc_url = home_url() . '?af_wuc-' . $post_id;
			update_post_meta($post_id, 'af_wuc_url', $af_wuc_url);
		}
	}

	public function af_wuc_get_product_variation_id( $product_id, $attributes ) {
		return ( new WC_Product_Data_Store_CPT() )->find_matching_product_variation(
			new WC_Product($product_id),
			$attributes
		);
	}

	public function af_wuc_metabox_field() {
		wp_nonce_field('af_wuc_nonce_action', 'af_wuc_nonce');

		include_once AF_WUC_PLUGIN_DIR . '/include/admin/metabox/af-wuc-meta-box-fields.php';
	}

	public function af_wuc_metabox_message() {
		wp_nonce_field('af_wuc_nonce_action', 'af_wuc_nonce');

		include_once AF_WUC_PLUGIN_DIR . '/include/admin/metabox/af-wuc-meta-box-custom-message.php';
	}

	public function af_wuc_metabox_coupon_error() {
		wp_nonce_field('af_wuc_nonce_action', 'af_wuc_nonce');
		?>
		<p><?php echo esc_html__('Coupons are expired or unavailable. Kindly check and update them before using this plugin.', 'addify-advanced-url-coupons'); ?></p> <br>
		<a href="<?php echo esc_url(admin_url('edit.php?post_type=shop_coupon')); ?>" class="button">View Coupons</a>
		<style>
			#submitdiv {
				display: none;
			}
		</style>
		<?php
	}


	public function af_wuc_live_product_search() {

		$nonce = isset($_POST['nonce']) ? sanitize_text_field(wp_unslash($_POST['nonce'])) : 0;

		if (!wp_verify_nonce($nonce, 'af_wuc_action')) {

			die(esc_html__('Failed ajax security check!', 'addify-advanced-url-coupons'));

		}

		if (isset($_POST['q']) && '' !== $_POST['q']) {

			$pro = sanitize_text_field(wp_unslash($_POST['q']));
		} else {
			$pro = '';
		}
		$product_type = wc_get_product_types();
		unset($product_type['grouped']);

		$data_array = array();
		$args       = array(
			'orderby'      => 'name',
			'order'        => 'asc',
			'post_type'    => array( 'product' ),
			'post_status'  => 'publish',
			'out_of_stock' => false,
			'numberposts'  => -1,
			's'            => $pro,
			'type'         => array_keys($product_type),
		);
		$pros       = wc_get_products($args);
		// print_r($pros);
		if (!empty($pros)) {
			foreach ($pros as $key => $proo) {

				$title = ( mb_strlen($proo->get_name()) > 50 ) ? mb_substr($proo->get_name(), 0, 49) . '...' : $proo->get_name();

				$data_array[] = array(
					'id'   => $proo->get_id(),
					'name' => $title,
				); // array( Post ID, Post Title, ).

			}
		}
		echo wp_json_encode($data_array);
		die();
	}

	public function af_wuc_get_product_attributes() {

		$nonce = isset($_POST['nonce']) ? sanitize_text_field(wp_unslash($_POST['nonce'])) : 0;

		if (!wp_verify_nonce($nonce, 'af_wuc_action')) {
			die(esc_html__('Failed ajax security check!', 'addify-advanced-url-coupons'));
		}

		$af_wuc_product_id = isset($_POST['product_id']) ? sanitize_text_field(wp_unslash($_POST['product_id'])) : '';

		$af_wuc_product                     = wc_get_product($af_wuc_product_id);
		$af_wuc_attributes                  = '';
		$af_wuc_single_product_max_quantity = '';
		$af_wuc_product_attributes          = array();
		$af_wuc_is_variable                 = '';

		if ('variable' === $af_wuc_product->get_type()) {
			foreach ($af_wuc_product->get_available_variations() as $key => $af_wuc_single_variation_product) {
				$af_wuc_product_attributes[] = $af_wuc_single_variation_product['attributes'];
			}
			foreach ($af_wuc_product->get_attributes() as $key => $attribute) {
				$af_wuc_attributes .= '<div class="af_wuc_attribute_inputs">';
				$af_wuc_attributes .= '<strong>' . esc_attr(wc_attribute_label($attribute['name'])) . ': &nbsp;</strong>';
				$af_wuc_attributes .= '<select class="af_wuc_product_attribute" name="af_wuc_product_attribute[' . esc_attr($af_wuc_product_id) . '][' . esc_attr(strtolower($key)) . ']" data-label="' . esc_attr(strtolower($key)) . '" >';
				$af_wuc_attributes .= '<option value="' . esc_attr('Choose an option') . '">' . esc_html__('Choose an option', 'addify-advanced-url-coupons') . '</option>';
				foreach ($attribute['options'] as $term_key => $term_id) {
					$term = get_term($term_id);
					if ($term) {
						$af_wuc_attributes .= '<option value="' . esc_attr($term->slug) . '">' . esc_html($term->name) . '</option>';
					} else {
						$af_wuc_attributes .= '<option value="' . esc_attr($term_id) . '">' . esc_html($term_id) . '</option>';
					}

				}
				$af_wuc_attributes .= '</select></div><br>';
			}
			$af_wuc_is_variable = 'disabled';
		} else {
			// Construct the availability message string separately
			$availability_class = ucwords(str_replace('-', ' ', $af_wuc_product->get_availability()['class']));
			
			// Concatenate the translated part and the dynamic value
			$af_wuc_message = esc_html__('Availability: ', 'addify-advanced-url-coupons') . esc_html($availability_class);

			if ('outofstock' !== $af_wuc_product->get_stock_status()) {
				$af_wuc_manage_stock = get_post_meta($af_wuc_product->get_id(), '_manage_stock', true);

				if ($af_wuc_product->get_sold_individually()) {
					$af_wuc_single_product_max_quantity = 1;
					// Translators: %d is the available quantity for a single product
					$af_wuc_message .= '<br>' . esc_html__('Available quantity= ', 'addify-advanced-url-coupons') . $af_wuc_single_product_max_quantity;
				} elseif ('yes' === $af_wuc_manage_stock) {
					$af_wuc_single_product_max_quantity = $af_wuc_product->get_stock_quantity();
					// Translators: %d is the available quantity for a product
					$af_wuc_message .= '<br>' . esc_html__('Available quantity= ', 'addify-advanced-url-coupons') . $af_wuc_single_product_max_quantity;
				}
			} else {
				$af_wuc_product_quantity_error_class   = 'af_wuc_prod_error';
				$af_wuc_product_quantity_input_disable = 'disabled';
			}
		}

		$product_name     = esc_html($af_wuc_product->get_name()); // Escape the product name
		$af_quantity_prod = '<tr><th class="af-wuc-table-td"><strong><a href="' . esc_url(get_edit_post_link($af_wuc_product_id)) . '">' . $product_name . '</a></strong></th><td class="af-wuc-table-td"><input type="number" name="af_wuc_products_quantity[' . esc_attr($af_wuc_product_id) . ']" required="required" min="1" max="' . esc_attr($af_wuc_single_product_max_quantity) . '" value="1" ' . esc_attr($af_wuc_is_variable) . ' class="af_wuc_prod_quantity af_wuc_prod_quantity_' . esc_attr($af_wuc_product_id) . ' " ' . $af_wuc_product_quantity_input_disable . '></td><td class="af-wuc-table-td" data-product_id="' . esc_attr($af_wuc_product_id) . '" data-product_variations="' . esc_attr(wp_json_encode($af_wuc_product_attributes)) . '">' . $af_wuc_attributes . '<div class="af_wuc_message ' . $af_wuc_product_quantity_error_class . '">' . $af_wuc_message . '</div></td><td class="af-wuc-table-td"><i class="fa fa-trash af_wuc_close_button"  title="Remove this Product"></i></td></tr>';

		wp_send_json_success(array( 'af_wuc_message' => $af_quantity_prod ));

		// die();
	}
	public function af_wuc_get_stock_quantity() {

		$nonce = isset($_POST['nonce']) ? sanitize_text_field(wp_unslash($_POST['nonce'])) : 0;

		if (!wp_verify_nonce($nonce, 'af_wuc_action')) {
			die(esc_html__('Failed ajax security check!', 'addify-advanced-url-coupons'));
		}

		$af_wuc_product_id = isset($_POST['product_id']) ? sanitize_text_field(wp_unslash($_POST['product_id'])) : '';

		$product_id             = isset($_POST['product_id']) ? absint($_POST['product_id']) : 0;
		$selected_variations    = isset($_POST['attributes']) ? sanitize_meta('', wp_unslash($_POST['attributes']), '') : array();
		$product_found          = false;
		$selected_variation_qty = '';
		$af_wuc_stock_status    = '';
		$af_wuc_manage_stock    = false;
		$af_wuc_message         = '';

		if (count($selected_variations)) {
			$product = wc_get_product($product_id);
			foreach ($product->get_available_variations() as $variation) {
				$match = true;
				foreach ($selected_variations as $key => $value) {
					if (!empty($variation['attributes'][ $key ])) {
						if ($variation['attributes'][ $key ] !== $value) {
							$match = false;
							break;
						}
					}
				}
				if ($match) {
					$variable_product = wc_get_product($variation['variation_id']);
					$product_found    = true;
					
					// Generate availability message (handle dynamic part outside esc_html__)
					$availability_class = ucwords(str_replace('-', ' ', $variable_product->get_availability()['class']));
					$af_wuc_message     = esc_html__('Availability: ', 'addify-advanced-url-coupons') . $availability_class;
					
					$af_wuc_stock_status = $variable_product->get_stock_status();
					$af_wuc_manage_stock = get_post_meta($variable_product->get_id(), '_manage_stock', true);
					
					if ('outofstock' !== $af_wuc_stock_status) {
						if ($variable_product->get_sold_individually()) {
							$selected_variation_qty = 1;
							$af_wuc_message        .= '<br>' . esc_html__('Available quantity = ', 'addify-advanced-url-coupons') . $selected_variation_qty;
						} elseif ('yes' === $af_wuc_manage_stock) {
							$selected_variation_qty = $variable_product->get_stock_quantity();
							$af_wuc_message        .= '<br>' . esc_html__('Available quantity = ', 'addify-advanced-url-coupons') . $selected_variation_qty;
						}
					}
					break;
				}


			}
			if (empty($selected_variation_qty)) {
				if ('outofstock' !== $af_wuc_stock_status && 'outofstock' !== $product->get_stock_status()) {
					if ('yes' === get_post_meta($product_id, '_manage_stock', true)) {
						$selected_variation_qty = $product->get_stock_quantity();
						
						// Translators: %d is the available quantity of the product.
						$message = sprintf(__('Available quantity = %d', 'addify-advanced-url-coupons'), $selected_variation_qty);
						
						$af_wuc_message .= '<br>' . esc_html($message);
					}
				}
			}


		}


		if (!isset($variable_product)) {
			$af_wuc_message         = esc_html__('This variation is not available ', 'addify-advanced-url-coupons');
			$selected_variation_qty = '';
		}

		wp_send_json_success(
			array(
				'product_found'       => $product_found,
				'af_wuc_quantity'     => $selected_variation_qty,
				'af_wuc_message'      => $af_wuc_message,
				'af_wuc_stock_status' => $af_wuc_stock_status,
				'af_wuc_manage_stock' => $af_wuc_manage_stock,
			),
			200
		);
	}

	public function af_wuc_meta_box_fields_style() {
			global $post_type;

			$current_screen = get_current_screen();
		   
			$afCurrent_tab =  filter_input(INPUT_GET, 'tab', FILTER_SANITIZE_SPECIAL_CHARS);

		if (( 'woocommerce_page_wc-settings' == $current_screen->id && 'woocommerce_url_coupon_tab' === $afCurrent_tab ) || 'af_wuc' === $current_screen->id || 'af_wuc' === $post_type) {

			wp_enqueue_style('af_wuc_', AF_WUC_URL . '/assets/css/af-wuc-admin-styling.css', false, '1.0.0');
			   
			wp_enqueue_style('af_wuc_font-awesome', AF_WUC_URL . '/assets/css/fontawesome/css/all.min.css', false, '6.0.0-beta3');

			wp_enqueue_script('addify-advanced-url-coupons_', AF_WUC_URL . '/assets/js/af-wuc-admin-script.js', array( 'jquery' ), '1.0.0', false);

			// Prepare nonce for AJAX requests
			$af_wuc_script_link_array = array(
				'ajax_url'               => admin_url('admin-ajax.php'),
				'af_wuc_form_data_nonce' => wp_create_nonce('af_wuc_action'),
			);
			wp_localize_script('addify-advanced-url-coupons_', 'af_wuc_link_obj', $af_wuc_script_link_array);
		}
	}

	public function af_wuc_get_sections() {
		global $current_section;

		$tab_id = 'woocommerce_url_coupon_tab';

		$sections = array(
			'section_general_setting_wuc' => esc_html__('Settings', 'addify-advanced-url-coupons'),
		);

		$array_keys      = array_keys($sections);
		$current_section = empty($current_section) ? 'section_general_setting_wuc' : $current_section;
		?>
		<ul class="subsubsub">
			<?php
			foreach ($sections as $id => $label) {
				// Output the list items with proper sanitization
				echo wp_kses_post(
					'<li><a href="' . admin_url('admin.php?page=wc-settings&tab=' . $tab_id . '&section=' . sanitize_title($id)) . '" class="' . ( $current_section === $id ? 'current' : '' ) . '">' . esc_html($label) . '</a> ' . ( end($array_keys) === $id ? '' : '|' ) . ' </li>'
				);
			}
			?>
		</ul>

		<br class="clear" />
		<?php
	}//end af_wuc_get_sections()


	public function af_wuc_add_settings_tab( $settings_tabs ) {
		$settings_tabs['woocommerce_url_coupon_tab'] = esc_html__('Advanced URL Coupons', 'addify-advanced-url-coupons');

		return $settings_tabs;
	}//end af_wuc_add_settings_tab()

	public function af_wuc_settings_tab() {
		// Call settings function
		$settings = $this->get_custom_settings();
		WC_Admin_Settings::output_fields($settings);
		wp_nonce_field('af_wuc_nonce_action', 'af_wuc_nonce');
	}


	public function get_custom_settings() {

		global $current_section;

		$settings = array();
		if ('section_general_setting_wuc' === $current_section) {
			$settings = array(
				'section_general'     => array(
					'name' => esc_html__('General', 'addify-advanced-url-coupons'),
					'type' => 'title',
					'desc' => '',
					'id'   => 'wc_url_coupon_tab_section_general',
				),
				'coupons_en'          => array(
					'name'    => esc_html__('Hide Coupon Field', 'addify-advanced-url-coupons'),
					'type'    => 'checkbox',
					'default' => 'no',
					'tooltip' => esc_html__('Hide coupon nput field from cart page.', 'addify-advanced-url-coupons'),
					'desc'    => esc_html__('Hide coupon input field from cart page.', 'addify-advanced-url-coupons'),
					'id'      => 'wc_url_coupon_tab_coupons_field_en',
				),
				'hide_on_cart_page'   => array(
					'name'    => esc_html__('Hide Applied Coupon', 'addify-advanced-url-coupons'),
					'type'    => 'checkbox',
					'default' => 'no',
					'id'      => 'wc_url_coupon_tab_hide_on_cart_page',
					'desc'    => esc_html__('Hide applied coupon from cart page.', 'addify-advanced-url-coupons'),
					'tooltip' => esc_html__('Hide applied coupon from cart page.', 'addify-advanced-url-coupons'),
				),

				'hide_checkout_page'  => array(
					'type'    => 'checkbox',
					'default' => 'no',
					'id'      => 'wc_url_coupon_tab_hide_on_checkout_page',
					'tooltip' => esc_html__('Hide applied coupon from checkout page.', 'addify-advanced-url-coupons'),
					'desc'    => esc_html__('Hide applied coupon from checkout page.', 'addify-advanced-url-coupons'),
				),
				'General_prefix'      => array(
					'name'  => esc_html__('General Prefix', 'addify-advanced-url-coupons'),
					'type'  => 'text',
					'value' => !empty(get_option('wc_url_general_prefix')) ? get_option('wc_url_general_prefix') : 'af_wuc',
					'id'    => 'wc_url_general_prefix',
					'desc'  => esc_html__('Add custom prefix/slug.', 'addify-advanced-url-coupons'),

				),
				'redirect_to_page'    => array(
					'name'     => esc_html__('Redirect to ', 'addify-advanced-url-coupons'),
					'type'     => 'select',
					'default'  => 'cart_page',
					'options'  => array(
						'cart_page'     => 'Redirect to cart page',
						'checkout_page' => 'Redirect to checkout page',
					),
					'id'       => 'wc_url_coupon_tab_redirect_to_page',
					'desc_tip' => true,
					'desc'     => esc_html__('Default it will be the cart Page', 'addify-advanced-url-coupons'),
				),
				'country'             => array(
					'name'              => esc_html__('Country Restrictions', 'addify-advanced-url-coupons'), // Corrected domain
					'type'              => 'multiselect',
					'label'             => esc_html__('Select Country', 'addify-advanced-url-coupons'), // Corrected domain
					'options'           => WC()->countries->countries,
					'custom_attributes' => array( 'data-placeholder' => esc_html__('Choose countries', 'addify-advanced-url-coupons') ), // Corrected domain
					'id'                => 'wc_af_wuc_country_input',
					'class'             => 'af_wuc_select_countries',
					'desc_tip'          => true,
					'desc'              => esc_html__('The functionality will be applied only to the selected countries. If no countries are selected, it will be available for all countries.', 'addify-advanced-url-coupons'), 
				),


				'section_general_end' => array(
					'type' => 'sectionend',
					'id'   => 'wc_url_coupon_tab_section_general_end',
				),
			);
		}

		return $settings;
	}//end get_custom_settings()

	// Add custom field below expiry date in coupon settings



	public function af_wuc_update_settings() {

		global $current_section;
		$tab_id          = 'woocommerce_url_coupon_tab';
		$current_section = $current_section ? $current_section : 'section_general_setting_wuc';

		// Call settings function
		$settings = $this->get_custom_settings();
		if ($current_section) {
			
			WC_Admin_Settings::save_fields($settings);

			/*
			 * Saving options values
			 *
			 * @since 1.0.0
			 *
			 */
			do_action('woocommerce_update_options_' . $tab_id . '_' . $current_section);
		}
	}//end af_wuc_update_settings()
}//end class

if (class_exists('AF_WUC_Admin')) {
	new AF_WUC_Admin();
}

?>