<?php
if (!defined('ABSPATH')) {
	exit;
}
class AF_WUC_Front {

	public function __construct() {

		add_action('wp_loaded', array( $this, 'af_wuc_add_products_to_cart_function' ));
		add_filter('woocommerce_coupons_enabled', array( $this, 'af_wuc_hide_coupon_field_on_cart' ));
		add_filter('woocommerce_update_cart_validation', array( $this, 'af_wuc_validate_products_quantity_in_cart' ), 10, 4);

		add_action('wp_enqueue_scripts', array( $this, 'enqueue_af_wuc_scripts' ));

		add_action('woocommerce_before_cart', array( $this, 'af_wuc_check_products_quantity_in_cart_function' ));
		add_action('woocommerce_remove_cart_item', array( $this, 'af_wuc_remove_products_from_cart_function' ), 10, 2);
		add_action('woocommerce_removed_coupon', array( $this, 'af_wuc_remove_products_from_cart_on_coupon_removal_function' ), 10, 2);
		add_action('woocommerce_proceed_to_checkout', array( $this, 'af_wuc_woocommerce_proceed_to_checkout' ));
		add_action('woocommerce_checkout_order_processed', array( $this, 'af_wuc_woocommerce_checkout_order_processed' ), 10, 3);
	}
	public function enqueue_af_wuc_scripts() {
		wp_enqueue_script('addify_wuc_front', AF_WUC_URL . '/assets/js/af-wuc-front-script.js', array( 'jquery' ), '1.0.0', false);
	}


	public function af_wuc_woocommerce_proceed_to_checkout() {

		if (isset(WC()->cart) && count(WC()->cart->get_applied_coupons()) < 1) {
			foreach (WC()->cart->get_cart() as $cart_item) {
				if (isset($cart_item['af_wuc_url_post'])) {
					unset($cart_item['af_wuc_url_post']);
				}
			}
		}
	}
	public function af_wuc_woocommerce_checkout_order_processed( $order_id, $posted_data, $order ) {
		if (isset(WC()->cart)) {
			$af_wuc_url_post_ids = array();
			foreach (WC()->cart->get_cart() as $cart_item) {
				if (isset($cart_item['af_wuc_url_post'])) {
					foreach ($cart_item['af_wuc_url_post'] as $af_wuc_url_post_id => $af_wuc_url_post) {
						if (!in_array((int) $af_wuc_url_post_id, $af_wuc_url_post_ids, true)) {
							$af_wuc_url_post_ids[] = (int) $af_wuc_url_post_id;
						}
					}
				}
			}
			if (!empty($af_wuc_url_post_ids)) {
				foreach ($af_wuc_url_post_ids as $af_wuc_url_post_id) {
					$af_wuc_order_new_post_id = wp_insert_post(
						array(
							'post_type'   => 'af_wuc_orders',
							'post_status' => 'publish',
							'post_parent' => $af_wuc_url_post_id,
						)
					);
					update_post_meta($af_wuc_order_new_post_id, 'af_wuc_order_id', $order_id);
				}
			}
		}
	}

	public function af_wuc_hide_coupon_field_on_cart( $enabled ) {
		// Enqueue CSS and JS only if options are set
		if (!empty(get_option('wc_url_coupon_tab_hide_on_checkout_page')) && 'yes' === get_option('wc_url_coupon_tab_hide_on_checkout_page')) {
			add_action('wp_footer', function () {
				echo '<style type="text/css">
	                .woocommerce-checkout-review-order-table tr.cart-discount,
	                .wp-block-woocommerce-checkout-order-summary-block .wc-block-components-totals-discount {
	                    display: none !important;
	                }
	            </style>';
				echo '<script type="text/javascript">
	                jQuery(document).ready(function ($) {
	                    setTimeout(function () {
	                        $(".woocommerce-checkout-review-order .cart-discount").remove();
	                        $(".wp-block-woocommerce-cart-order-summary-block  .wc-block-components-totals-discount").remove();
	                    }, 10000);
	                });
	            </script>';
			});
		}

		if (!empty(get_option('wc_url_coupon_tab_hide_on_cart_page')) && 'yes' === get_option('wc_url_coupon_tab_hide_on_cart_page')) {
			add_action('wp_footer', function () {
				echo '<style type="text/css">
	                .shop_table_responsive tr.cart-discount,
	                .wp-block-woocommerce-cart-order-summary-block .wc-block-components-totals-discount {
	                    display: none !important;
	                }
	            </style>';
				echo '<script type="text/javascript">
	                document.addEventListener("DOMContentLoaded", function () {
	                    var cartPageElement = document.querySelector(".shop_table_responsive tr.cart-discount");
	                    var blockDiscountElement = document.querySelector(".wp-block-woocommerce-cart-order-summary-block .wc-block-components-totals-discount");

	                    if (cartPageElement) {
	                        cartPageElement.remove();
	                    }
	                    if (blockDiscountElement) {
	                        blockDiscountElement.remove();
	                    }
	                });
	            </script>';
			});
		}

		if (!empty(get_option('wc_url_coupon_tab_coupons_field_en')) && 'yes' === get_option('wc_url_coupon_tab_coupons_field_en')) {
			add_action('wp_footer', function () {
				echo '<style type="text/css">
	                div.coupon,
	                .wp-block-woocommerce-cart-order-summary-coupon-form-block {
	                    display: none !important;
	                }
	            </style>';
				echo '<script type="text/javascript">
	                document.addEventListener("DOMContentLoaded", function () {
	                    var couponElement = document.querySelector("div.coupon");
	                    var blockCouponFormElement = document.querySelector(".wc-block-components-coupon-form");

	                    if (couponElement) {
	                        couponElement.remove();
	                    }
	                    if (blockCouponFormElement) {
	                        blockCouponFormElement.remove();
	                    }
	                });
	            </script>';
			});
		}

		return $enabled;
	}

	public function af_wuc_add_products_to_cart_function() {
		$arg = get_posts(
			array(
				'numberposts' => -1,
				'post_type'   => array( 'af_wuc', 'shop_coupon' ),
				'post_status' => 'publish',
				'fields'      => 'ids',
			)
		);

		$general_prefix = !empty(get_option('wc_url_general_prefix')) ? get_option('wc_url_general_prefix') : 'af_wuc';
		$over_all_url   = isset($_GET) ? sanitize_meta('', $_GET, '') : array();

		if ('checkout_page' == get_option('wc_url_coupon_tab_redirect_to_page')) {
			$custom_redirect_url = wc_get_page_permalink('checkout');
		} else {
			$custom_redirect_url = wc_get_page_permalink('cart');

		}

		foreach ($arg as $rule_id) {
			$af_wuc_prefix = !empty(get_post_meta($rule_id, 'af_wuc_prefix', true)) ? get_post_meta($rule_id, 'af_wuc_prefix', true) : $general_prefix;
			// echo " ==> <br>" . $af_wuc_prefix . '-' . $rule_id ;
			// var_dump( !isset($over_all_url[$af_wuc_prefix . '-' . $rule_id]) );

			if (!isset($over_all_url[ $af_wuc_prefix . '-' . $rule_id ]) && !in_array($af_wuc_prefix . '-' . $rule_id, (array) $over_all_url)) {
				continue;
			}


			if ('shop_coupon' == get_post_type($rule_id)) {
				// Get the coupon code
				$coupon_code = get_the_title($rule_id);

				// Initialize an array to store messages
				$messages = array(
					'success' => array(),
					'error'   => array(),
				);

				// Check if the current user's country is valid for this coupon
				$country_error = false;
				if (!empty(get_option('wc_af_wuc_country_input')) && !empty($this->af_wuc_current_user_country_name()) && !in_array($this->af_wuc_current_user_country_name(), (array) get_option('wc_af_wuc_country_input'), true)) {
					$messages['error'][] = esc_html__('This coupon is not valid for your country.', 'addify-advanced-url-coupons');
					$country_error       = true;
				}

				// Proceed only if there's no country error
				if (!$country_error) {
					// Apply the coupon and get the result
					$coupon_applied = WC()->cart->apply_coupon($coupon_code);

					// Check if the coupon was applied successfully
					if (true === $coupon_applied) {
						// Get the success messages from WooCommerce session messages
						$success_messages = wc_get_notices('success');
						if (!empty($success_messages)) {
							foreach ($success_messages as $msg) {
								$messages['success'][] = $msg['notice']; // Store each success message
							}
						}
					} else {
						// Get the error messages from WooCommerce session messages
						$error_messages = wc_get_notices('error');
						if (!empty($error_messages)) {
							foreach ($error_messages as $msg) {
								$messages['error'][] = $msg['notice']; // Store each error message
							}
						}
					}
				}

				// Clear the WooCommerce notices
				wc_clear_notices();

				// Store the messages array in the session
				WC()->session->set('coupon_messages', $messages);
			} else {



			global $wp;

			$af_wuc_post_id    = $rule_id;
			$redirect_page_url = $custom_redirect_url . '?' . $af_wuc_prefix . '-' . $rule_id;
				if (!isset($_GET['af_wuc_cart_nonce']) || empty($_GET['af_wuc_cart_nonce'])) {
					$url     = wp_nonce_url($redirect_page_url, 'af_wuc_cart_nonce', 'af_wuc_cart_nonce');
					$new_url = str_replace('amp;', '', $url);
					wp_safe_redirect($new_url, 301);
					exit;
				}
			$nonce = sanitize_text_field(wp_unslash($_GET['af_wuc_cart_nonce']));

				if (!wp_verify_nonce($nonce, 'af_wuc_cart_nonce')) {
					wc_add_notice(esc_html__('Security voilation.', 'addify-advanced-url-coupons'), 'error');
					wp_safe_redirect(home_url($wp->request), '302');
					exit;
				}

			$af_wuc_post = get_post($rule_id);
				if ($af_wuc_post) {
			

					$af_wuc_products_stored_quantity = (array) get_post_meta($rule_id, 'af_wuc_products_quantity', true);
					$af_wuc_coupon_url_message       = get_post_meta($rule_id, 'af_wuc_coupon_url_message', true);
					$af_wuc_coupon_stored            = get_post_meta($rule_id, 'af_wuc_coupon', true);
			

					$af_wuc_product_variation_stored = get_post_meta($rule_id, 'af_wuc_product_variation', true);

					$error                   = 0;
					$q_error                 = 0;
					$coupon_error            = 0;
					$country_error           = false;
					$product_variation_error = array();

					if (!empty(get_option('wc_af_wuc_country_input')) && !empty($this->af_wuc_current_user_country_name()) && !in_array($this->af_wuc_current_user_country_name(), (array) get_option('wc_af_wuc_country_input'), true)) {
						//wc_add_notice(esc_html__('This link is not valid for your country', 'addify-advanced-url-coupons'), 'error');
						$country_error = true;
					}

					foreach ((array) $af_wuc_products_stored_quantity as $af_wuc_product_id => $af_wuc_product_quantity) {
						$af_wuc_product = wc_get_product($af_wuc_product_id);

						if ($af_wuc_product) {
							if (isset(WC()->cart)) {

								$product_variation_id = isset($af_wuc_product_variation_stored[ $af_wuc_product_id ]['variation_id']) ? $af_wuc_product_variation_stored[ $af_wuc_product_id ]['variation_id'] : 0;
								foreach (WC()->cart->get_cart() as $cart_item) {
									if (!isset($cart_item['product_id'])) {
										continue;
									}
									$cart_product = $cart_item['data'];

									if (( $af_wuc_product_id === $cart_product->get_id() || $product_variation_id === $cart_product->get_id() ) && isset($cart_item['af_wuc_url_post'])) {
										$af_wuc_product = $product_variation_id ? wc_get_product($product_variation_id) : $af_wuc_product;
										if ('yes' === get_post_meta($cart_item['data']->get_id(), '_manage_stock', true)) {
											if ($af_wuc_product_quantity + $cart_item['quantity'] > $af_wuc_product->get_stock_quantity()) {
												
												wc_add_notice(
													sprintf(
														// Translators: %s is the product name.
														esc_html__('%s stock is low', 'addify-advanced-url-coupons'), $af_wuc_product->get_name()),
													'error'
												);


												++$q_error;
											}
										}
									}
								}
							}

							if ('yes' === get_post_meta($af_wuc_product->get_id(), '_manage_stock', true)) {
								if ($af_wuc_product->get_stock_quantity() < $af_wuc_product_quantity) {
									wc_add_notice(
												sprintf(
													// Translators: %s is the product name.
													esc_html__('%s stock is low', 'addify-advanced-url-coupons'), $af_wuc_product->get_name()),
												'error'
											);
									++$q_error;
								}
							} elseif ('outofstock' === $af_wuc_product->get_stock_status()) {
								$product_name = $af_wuc_product->get_name();
								$message      = sprintf(
									// Translators: %s is the product name
									esc_html__('The Product %s used in the URL is out of stock', 'addify-advanced-url-coupons'), $product_name);
								wc_add_notice($message, 'error');

							}
						} else {
							++$error;
						}
					}

					if ($error) {
						wc_add_notice(esc_html__('Some products are not found in the store.', 'addify-advanced-url-coupons'), 'error');
					}

					$af_coupon_post = get_post($af_wuc_coupon_stored);
					$coupon_details = new WC_Coupon($af_coupon_post->post_title);

					if ($error || $q_error || $country_error) {
						wc_add_notice(esc_html__('This link is not valid for your country.', 'addify-advanced-url-coupons'), 'error');
						return;
					} else {
						$cart                            = WC()->cart;
						$af_wuc_product_messages         = array();
						$af_wuc_cart_item_keys           = array();
						$af_wuc_cart_item_quantity_reset = array();

						foreach ((array) $af_wuc_products_stored_quantity as $product_id => $af_wuc_single_product_quantity) {
							$product              = wc_get_product($product_id);
							$product_variation_id = isset($af_wuc_product_variation_stored[ $product_id ]['variation_id']) ? $af_wuc_product_variation_stored[ $product_id ]['variation_id'] : 0;
							$product_variation    = isset($af_wuc_product_variation_stored[ $product_id ]['attributes']) ? $af_wuc_product_variation_stored[ $product_id ]['attributes'] : array();
							$is_product_in_cart   = false;

							$af_wuc_url_post = array();

							$af_wuc_url_post[ $af_wuc_post_id ]['product_min_quantity'][] = array(
								'af_wuc_product_id'       => $product_id,
								'af_wuc_variation_id'     => $product_variation_id,
								'af_wuc_product_quantity' => $af_wuc_single_product_quantity,
							);
							$af_wuc_url_post[ $af_wuc_post_id ]['af_wuc_coupon'][]        = $coupon_details->get_code();
							if (isset(WC()->cart)) {
								foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {
									if (!isset($cart_item['product_id'])) {
										continue;
									}

									$cart_product = $cart_item['data'];

									if (( $product_id === $cart_product->get_id() || $product_variation_id === $cart_product->get_id() ) && isset($cart_item['af_wuc_url_post'])) {

										$af_wuc_cart_item_quantity_reset[ $cart_item_key ]['quantity'] = (int) $cart_item['quantity'];

										$current_quantity = (int) $cart_item['quantity'] + (int) $af_wuc_single_product_quantity;

										$product = $product_variation_id ? wc_get_product($product_variation_id) : $cart_product;

										if ('yes' === get_post_meta($product->get_id(), '_manage_stock', true)) {
											$current_quantity = $current_quantity < $product->get_stock_quantity() ? $current_quantity : $product->get_stock_quantity();
										}

										WC()->cart->set_quantity($cart_item_key, $current_quantity);

										$af_wuc_cart_item_quantity_reset[ $cart_item_key ]['af_wuc_url_post_id'] = $af_wuc_post_id;
										if (isset(WC()->cart->cart_contents[ $cart_item_key ]['af_wuc_url_post'])) {
											WC()->cart->cart_contents[ $cart_item_key ]['af_wuc_url_post'] += $af_wuc_url_post;
										} else {
											WC()->cart->cart_contents[ $cart_item_key ]['af_wuc_url_post'] = $af_wuc_url_post;
										}

										$is_product_in_cart = true;

										// Add message to array
										$product_name = $product->get_name(); // Get the product name
										$message      = sprintf(
											// Translators: %s is the name of the product
											esc_html__( '"%s" quantity is updated', 'addify-advanced-url-coupons' ), $product_name );
										$af_wuc_product_messages[] = $message;

									}
								}
							}


							if (!$is_product_in_cart) {

								$product = $product_variation_id ? wc_get_product($product_variation_id) : $product;
								if ('yes' === get_post_meta($product->get_id(), '_manage_stock', true)) {
									$af_wuc_single_product_quantity = $af_wuc_single_product_quantity < $product->get_stock_quantity() ? $af_wuc_single_product_quantity : $product->get_stock_quantity();
								}

								if (!$this->af_wuc_check_product_variation($product, $product_variation)) {
									$product_name = $product->get_name();
$error_message                                    = sprintf(
	// Translators: %s is the product name
	esc_html__('The selected variation of "%s" is not available', 'addify-advanced-url-coupons'), $product_name);
$product_variation_error[] = $error_message;

								}
								$af_wuc_cart_item_keys[] = WC()->cart->add_to_cart(
								$product_id,
								$af_wuc_single_product_quantity,
								$product_variation_id,
								$product_variation,
								array(
									'af_wuc_url_post' => $af_wuc_url_post,
								)
								);


								if ('outofstock' !== $product->get_stock_status()) {
									$af_wuc_product_messages[] = sprintf( 
										// Translators: %s is the name of the product
										esc_html__( '"%s" has been added to your cart', 'addify-advanced-url-coupons' ), $product->get_name() );

								}
							}
						}


						if (!empty($product_variation_error)) {
							foreach ($af_wuc_cart_item_quantity_reset as $af_wuc_cart_item_key => $af_wuc_cart_item_reset) {
								unset(WC()->cart->cart_contents[ $af_wuc_cart_item_key ]['af_wuc_url_post'][ $af_wuc_cart_item_reset['af_wuc_url_post_id'] ]);
								WC()->cart->set_quantity($af_wuc_cart_item_key, $af_wuc_cart_item_reset['quantity']);
							}
							foreach ($af_wuc_cart_item_keys as $af_wuc_cart_item_key) {
								WC()->cart->remove_cart_item($af_wuc_cart_item_key);
							}
							foreach ($product_variation_error as $af_wuc_product_error_message) {
								wc_add_notice(wp_kses_post($af_wuc_product_error_message), 'error');
							}

							wc_add_notice(esc_html__('The Url Coupon could not be used to add products and coupons as it has the above errors.', 'addify-advanced-url-coupons'), 'error');
							return;
						}

						$discounts    = new \WC_Discounts(WC()->cart);
						$coupon_valid = $discounts->is_coupon_valid($coupon_details);

						// Check if coupon is valid or has errors
						if ('boolean' !== gettype($coupon_valid)) {
							if ($coupon_valid->has_errors()) {
								++$coupon_error;
								foreach ($coupon_valid->get_error_messages() as $single_error_message) {
									$error_message = esc_html__('Coupon Error: ', 'addify-advanced-url-coupons') . wp_kses_post($single_error_message);
									wc_add_notice($error_message, 'error');  // Show error messages for invalid/expired coupon
								}
							}
						}

						if (!$coupon_error && !in_array($coupon_details->get_code(), WC()->cart->get_applied_coupons(), true)) {
							WC()->cart->apply_coupon($coupon_details->get_code());
						}

						if (!in_array($coupon_details->get_code(), WC()->cart->get_applied_coupons(), true) && !$coupon_error) {
							++$coupon_error;
							wc_add_notice(esc_html__('Coupon Error: The coupon could not be used in this cart.', 'addify-advanced-url-coupons'), 'error');
						}
						if ($coupon_error) {
							foreach ($af_wuc_cart_item_quantity_reset as $af_wuc_cart_item_key => $af_wuc_cart_item_reset) {
								unset(WC()->cart->cart_contents[ $af_wuc_cart_item_key ]['af_wuc_url_post'][ $af_wuc_cart_item_reset['af_wuc_url_post_id'] ]);
								WC()->cart->set_quantity($af_wuc_cart_item_key, $af_wuc_cart_item_reset['quantity']);
							}
							foreach ($af_wuc_cart_item_keys as $af_wuc_cart_item_key) {
								WC()->cart->remove_cart_item($af_wuc_cart_item_key);
							}

							wc_add_notice(esc_html__('The Url Coupon could not be used to add products and coupons as it has the above errors.', 'addify-advanced-url-coupons'), 'error');
							return;
						}
						if (!empty($af_wuc_product_messages)) {
							if (!empty(WC()->session)) {
								// Retrieve existing session messages
								$messages = WC()->session->get('af_wuc_messages', array());

								// Add product messages
								foreach ($af_wuc_product_messages as $af_wuc_product_message) {
									$messages[] = $af_wuc_product_message;
								}
								if (is_checkout()) {

									// Check if there is no coupon error and the coupon is applied
									if (empty($coupon_error) && in_array($coupon_details->get_code(), WC()->cart->get_applied_coupons(), true)) {
										// Check if the coupon is valid
										$discounts    = new \WC_Discounts(WC()->cart);
										$coupon_valid = $discounts->is_coupon_valid($coupon_details);
										if (true === $coupon_valid) {
											$coupon_success_message = esc_html__('Coupon applied successfully', 'addify-advanced-url-coupons');
											$messages[]             = $coupon_success_message;
										}
									}
								}
								if (!empty($af_wuc_coupon_url_message)) {
							
									$messages[] = ( $af_wuc_coupon_url_message );
								
								}


								  



								// Update the session with all the messages
								WC()->session->set('af_wuc_messages', $messages);
							}
						}


					}
				}

			WC()->cart->set_session();


			}

		$this->af_wuc_redirect_to_page();

		}
	}



	public function af_wuc_check_product_variation( $product, $selected_attributes ) {
		$product_attributes = $product->get_attributes();


		$product_attributes_with_prefix = array_combine(
			array_map(function ( $key ) {
				return 'attribute_' . $key;
			}, array_keys($product_attributes)),
			$product_attributes
		);
		foreach ($selected_attributes as $key => $selected_attribute) {
			if (!empty($product_attributes_with_prefix[ $key ]) && $product_attributes_with_prefix[ $key ] !== $selected_attribute) {
				return false;
			}
		}
		return true;
	}

	public function af_wuc_current_user_country_name() {
		$af_wuc_current_user_country_data = WC_Geolocation::geolocate_ip();
		return $af_wuc_current_user_country_data['country'];
	}

	public function af_wuc_redirect_to_page() {
		$af_wuc_redirect_url = !empty(get_option('wc_url_coupon_tab_redirect_to_page')) && get_option('wc_url_coupon_tab_redirect_to_page') === 'checkout_page' ? wc_get_checkout_url() : wc_get_cart_url();
		wp_safe_redirect($af_wuc_redirect_url, '302');
		exit;
	}

	public function af_wuc_validate_products_quantity_in_cart( $passed_validation, $cart_item_key, $cart_item, $quantity ) {

		if (isset($cart_item['af_wuc_url_post'])) {
			$af_wuc_min_quantity           = 0;
			$af_wuc_quantity_error_message = '';
			foreach ($cart_item['af_wuc_url_post'] as $data) {
				foreach ($data['product_min_quantity'] as $min_quantity) {
					if ($cart_item['product_id'] === $min_quantity['af_wuc_product_id'] && $cart_item['variation_id'] === $min_quantity['af_wuc_variation_id']) {
						$af_wuc_min_quantity += (int) $min_quantity['af_wuc_product_quantity'];
					}
				}
			}
			if ($quantity < $af_wuc_min_quantity) {
				WC()->cart->set_quantity($cart_item_key, $af_wuc_min_quantity);
				$af_wuc_quantity_error_message = sprintf(
					// Translators: %s is the name of the product
					esc_html__('The minimum quantity for the product %1$s, %2$d in this URL Coupon', 'addify-advanced-url-coupons'),
					$cart_item['data']->get_name(),
					$af_wuc_min_quantity
				);

			} elseif (!empty($cart_item['data']->get_stock_quantity())) {
				if ($quantity > $cart_item['data']->get_stock_quantity()) {
					$af_wuc_quantity_error_message = sprintf(
						// Translators: %s is the name of the product
						esc_html__('The maximum available quantity for the product %1$s, %2$d  in stock.', 'addify-advanced-url-coupons'),
						$cart_item['data']->get_name(),
						$cart_item['data']->get_stock_quantity()
					);

				}
			}
			if (!empty($af_wuc_quantity_error_message)) {
				$passed_validation = false;
				wc_add_notice($af_wuc_quantity_error_message, 'error');
			}
		}
		return $passed_validation;
	}

	public function af_wuc_check_products_quantity_in_cart_function() {

		if (isset(WC()->cart)) {
			$af_wuc_notices = array();
			foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {
				if (isset($cart_item['af_wuc_url_post'])) {
					$af_wuc_min_quantity = 0;
					foreach ($cart_item['af_wuc_url_post'] as $data) {
						foreach ($data['product_min_quantity'] as $min_quantity) {
							if ($cart_item['product_id'] === $min_quantity['af_wuc_product_id'] && $cart_item['variation_id'] === $min_quantity['af_wuc_variation_id']) {
								$af_wuc_min_quantity += (int) $min_quantity['af_wuc_product_quantity'];
							}
						}
					}

					if ($cart_item['quantity'] < $af_wuc_min_quantity) {
						WC()->cart->set_quantity($cart_item_key, $af_wuc_min_quantity);
						// Translators: %s is the name of the product
					$message = esc_html__('The minimum quantity for the product %1$s, %2$d in this URL Coupon', 'addify-advanced-url-coupons');

					// Use sprintf to replace the placeholders with actual values
					$af_wuc_notices[] = sprintf(
						// Translators: %s is the name of the product
						$message, $cart_item['data']->get_name(), $af_wuc_min_quantity);

					} elseif ('yes' === get_post_meta($cart_item['data']->get_id(), '_manage_stock', true)) {
						if ($cart_item['quantity'] > $cart_item['data']->get_stock_quantity()) {
							if (!empty($cart_item['data']->get_stock_quantity())) {
								$af_wuc_notices[] = sprintf(
									// Translators: %s is the name of the product
									esc_html__('The maximum available quantity for the product %1$s, %2$d in the Stock', 'addify-advanced-url-coupons'),
									$cart_item['data']->get_name(),
									$cart_item['data']->get_stock_quantity()
								);

							}  {
								// Translators: %s is the name of the product
								$af_wuc_notices[] = esc_html__( 'The stock of product %s is very low.', 'addify-advanced-url-coupons' );
								$af_wuc_notices[] = sprintf( 
									// Translators: %s is the name of the product
									$af_wuc_notices[ count($af_wuc_notices)-1 ], $cart_item['data']->get_name() );

							}
						}
					}

				}
			}

			if (!empty($af_wuc_notices)) {
				foreach ($af_wuc_notices as $af_wuc_notice) {
					wc_add_notice($af_wuc_notice, 'error');
				}
			}
		}
	}

	public function af_wuc_remove_products_from_cart_function( $cart_item_key, $cart ) {
		if (isset($cart->cart_contents[ $cart_item_key ]['af_wuc_url_post'])) {
			// Get all applied coupons
			$applied_coupons = WC()->cart->get_applied_coupons();

			// Loop through the applied coupons and remove them
			foreach ($applied_coupons as $coupon_code) {
				// Only remove coupons associated with the specific product
				if ($this->is_coupon_related_to_product($coupon_code, $cart_item_key)) {
					WC()->cart->remove_coupon($coupon_code);
					//wc_add_notice( esc_html__( 'Coupon "' . $coupon_code . '" has been removed due to product removal.', 'addify_wuc' ), 'notice' );
				}
			}
		}
	}

	public function af_wuc_remove_products_from_cart_on_coupon_removal_function( $coupon_code ) {
		$cart                  = WC()->cart;
		$cart_items_to_remove  = array();
		$removed_product_names = array(); // Array to store names of removed products

		foreach ($cart->get_cart() as $cart_item_key => $cart_item) {
			if (isset($cart_item['af_wuc_url_post'])) {
				$custom_data = $cart_item['af_wuc_url_post'];

				foreach ($custom_data as $post_id => $data) {
					if (in_array((string) $coupon_code, $data['af_wuc_coupon'], true)) {
						$coupon_index = array_search($coupon_code, $data['af_wuc_coupon'], true);

						unset($custom_data[ $post_id ]['af_wuc_coupon'][ $coupon_index ]);

						if (isset($custom_data[ $post_id ]['product_min_quantity'])) {
							unset($custom_data[ $post_id ]['product_min_quantity'][ $coupon_index ]);
						}

						if (empty($custom_data[ $post_id ]['af_wuc_coupon']) && empty($custom_data[ $post_id ]['product_min_quantity'])) {
							unset($custom_data[ $post_id ]);
						}
					}
				}

				if (empty($custom_data)) {
					$cart_items_to_remove[ $cart_item_key ] = $cart_item;
					// Collect the product name for notification
					if (isset($cart_item['data'])) {
						$removed_product_names[] = $cart_item['data']->get_name();
					}
				} else {
					$cart->cart_contents[ $cart_item_key ]['af_wuc_url_post'] = $custom_data;
				}
			}
		}

		foreach ($cart_items_to_remove as $cart_item_key => $cart_item) {
			$cart->remove_cart_item($cart_item_key);
		}

		// Display error message for removed products
		if (!empty($removed_product_names)) {
			$product_list = implode(', ', $removed_product_names);
			
			$message = sprintf(
				// Translators: %1$s is the coupon code, %2$s is the list of products removed.
				esc_html__('The following products have been removed as the coupon "%1$s" was removed from the cart: %2$s', 'addify-advanced-url-coupons'),
				esc_html($coupon_code),
				$product_list
			);
			wc_add_notice($message , 'error');


		}

		$cart->set_session();
	}


	// Helper function to check if a coupon is related to a product
	private function is_coupon_related_to_product( $coupon_code, $cart_item_key ) {
		$cart      = WC()->cart;
		$cart_item = $cart->cart_contents[ $cart_item_key ];

		if (isset($cart_item['af_wuc_url_post'])) {
			foreach ($cart_item['af_wuc_url_post'] as $data) {
				if (in_array($coupon_code, $data['af_wuc_coupon'], true)) {
					return true; // Coupon is related to this product
				}
			}
		}

		return false; // Coupon is not related
	}
}//end class

new AF_WUC_Front();
